"use strict";
const readline = require("readline");
const colors = require("ph-terminal-colors");
const icons = require("ph-terminal-icons");
/**
 * 命令行加载提示类
 */
class Spinner {
    /**
     * 构造命令行提示器
     * @param text 提示文本
     */
    constructor(text) {
        this.text = text;
        this.animSteps = {
            frames: ['|', '/', '—', '\\'],
            interval: 200,
        };
        this._interval = -1;
        this._frameIndex = 0;
        this.rl = readline.createInterface({
            input: process.stdin,
            output: process.stdout,
        });
        this._lastText = '';
    }
    /**
     * 清除之前的渲染
     */
    clear() {
        // 清除之前的渲染
        readline.clearLine(process.stdout, 0);
        // 移动光标到开始位置
        readline.moveCursor(process.stdout, -(this._lastText.length * 2 + 2), 0); // * 2 一个中文占两个字符
    }
    /**
     * 开启模拟加载动画
     */
    start() {
        this._render();
        if (this._interval === -1) {
            this._interval = setInterval(() => {
                if (this._frameIndex < this.animSteps.frames.length - 1) {
                    this._frameIndex++;
                }
                else {
                    this._frameIndex = 0;
                }
                this._render();
            }, this.animSteps.interval);
        }
    }
    /**
     * 停止动画，更改图标为 √，重置文本
     * @param {String} text 显示文本
     */
    succeed(text) {
        this._resetStatus('success', text);
    }
    /**
     * 停止动画，更改图标为 ×，重置文本
     * @param {String} text
     */
    fail(text) {
        this._resetStatus('error', text);
    }
    /**
     * 停止加载动画
     */
    stop() {
        clearInterval(this._interval);
        this.rl.close();
    }
    _resetStatus(status, text) {
        clearInterval(this._interval);
        this._interval = -1;
        this.clear();
        this._lastText = '';
        this.rl.write((icons[status || 'info'] || 'X') + ' ' + (text || this.text) + '\r\n');
    }
    /**
     * 重新渲染文本
     * @param text 待渲染的文本
     */
    _render(text) {
        this.clear();
        this._lastText = text || this.text;
        this.rl.write(colors.blue(this.animSteps.frames[this._frameIndex] + ' ' + this._lastText));
    }
}
module.exports = (text) => new Spinner(text);
